# dynamic_test_runner.py
import os
import tempfile
from typing import Callable, Awaitable, List, IO

import pytest

from bananalyzer.data.schemas import Example

Test = Callable[[], Awaitable[None]]


def generate_test(example: Example, headless: bool) -> str:
    return f"""
@pytest.mark.asyncio
async def test_{example.id.replace("-", "_")}() -> None:
    async with async_playwright() as p:
        browser = await p.chromium.launch(headless={headless})
        context = await browser.new_context()
        example = get_example_by_url("{example.url}")
         
        # The agent is imported into the global context prior to this call 
        result = await agent.run(context, example)
        for curr_eval in example.evals:
            assert curr_eval.eval_results(result)
"""


def write_load_agent_module(f: IO[str], file_path: str) -> None:
    f.write(
        f"""
import importlib.util
import sys
from pathlib import Path

def load_agent_module():
    path = Path("{file_path}")
    module_name = path.stem

    spec = importlib.util.spec_from_file_location(module_name, "{file_path}")
    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)
    globals()["agent"] = getattr(module, "agent")
load_agent_module()
"""
    )


def run_tests(tests: List[str], agent_file_path: str) -> int:
    """
    Create temporary test file, run it, and then delete it
    """
    with tempfile.NamedTemporaryFile(mode="w+", delete=False, suffix=".py") as f:
        f.write("import pytest\n")
        f.write("from bananalyzer.data.examples import get_example_by_url\n")
        f.write("from playwright.async_api import async_playwright\n\n")
        f.write("global test\n")
        f.write("test = 'test'\n")
        write_load_agent_module(f, agent_file_path)

        for test_content in tests:
            f.write(f"{test_content}\n\n")

        test_file_name = f.name

    try:
        return_code = pytest.main([test_file_name])
    finally:
        os.unlink(test_file_name)

    return return_code



from abc import abstractmethod, ABC
from typing import Dict, Union, Any, List

from playwright.async_api import BrowserContext

from bananalyzer.data.schemas import Example

AgentResult = Union[Dict[str, Any], List[Dict[str, Any]], str]


class AgentRunner(ABC):
    """
    Wrapper class clients must implement to run an agent against the evaluations
    """

    @abstractmethod
    async def run(
        self,
        browser_context: BrowserContext,
        eval_context: Example,
    ) -> AgentResult:
        pass



"""
Mapping of fetch_id to fetch schema to avoid duplicate schemas in examples.json
"""
fetch_goals = {
    "job_posting": "Return the provided information about the job posting. For salaries, provide the range as ${lower} - ${upper} if available, otherwise just provide ${salary}",
}

fetch_schemas = {
    "job_posting": {
        "job_id": "string",
        "job_title": "string",
        "job_category": "string",
        "date_posted": "string",
        "location": "string",
        "job_description": "string",
        "roles_and_responsibilities": "string",
        "qualifications": "string",
        "preferred_qualifications": "string",
        "benefits": "string",
        "salary": "string",
    },
    "manufacturing_commerce": {
        "mpn": "string",
        "alias_mpns": ["string"],
        "manufacturer": "string",
        "classifications": ["string"],
        "description": "string",
        "hero_image": "string",
        "series": "string",
        "lifecycle_status": "string",
        "country_of_origin": "string",
        "aecq_status": "string",
        "reach_status": "string",
        "rohs_status": "string",
        "export_control_class_number": "string",
        "packaging": "string",
        "power_rating": "string",
        "voltage_rating": "string",
        "mount_type": "string",
        "moisture_sensitivity_level": "string",
        "tolerance": "string",
        "inductance": "string",
        "capacitance": "string",
        "resistance": "string",
        "min_operating_temperature": "string",
        "max_operating_temperature": "string",
        "leadfree": "string",
        "termination_type": "string",
        "num_terminations": "int",
        "specs": [{"label": "string", "value": "string"}],
        "product_change_notification_documents": [
            {"url": "string", "filename": "string"}
        ],
        "reach_compliance_documents": [{"url": "string", "filename": "string"}],
        "rohs_compliance_documents": [{"url": "string", "filename": "string"}],
        "datasheets": [{"url": "string", "filename": "string"}],
        "specsheets": [{"url": "string", "filename": "string"}],
        "suggested_alternative_mpns": ["string"],
    },
}



import asyncio

from playwright.async_api import BrowserContext

from bananalyzer.data.schemas import Example
from bananalyzer.runner.agent_runner import AgentResult, AgentRunner


class NullAgentRunner(AgentRunner):
    """
    A test agent class that just returns an empty string
    """

    async def run(
        self,
        context: BrowserContext,
        example: Example,
    ) -> AgentResult:
        page = await context.new_page()
        await page.goto(example.get_static_url())
        await asyncio.sleep(0.5)
        return example.evals[0].expected



from abc import ABC, abstractmethod
from typing import Dict, Literal, Union, Any, List, Optional

from pydantic import BaseModel, Field, model_validator

from bananalyzer.data.fetch_schemas import fetch_schemas

GoalType = Literal[
    "fetch",  # Scrape specific JSON information from a single page. Does not require navigation
    "click",  # Make a single click on a page
    "navigate",  # Travel to a new page
    "search",  # Search for the answer to a specific query
    "multiple",  # Perform multiple intents
]


class Eval(BaseModel, ABC):
    """
    Base class for all evals. Evals are used to determine if an action or result is correct
    """

    @abstractmethod
    def eval_action(self, action: str) -> bool:
        raise NotImplementedError("eval_action not implemented")

    @abstractmethod
    def eval_result(self, result: str) -> bool:
        raise NotImplementedError("eval_result not implemented")


class JSONEval(BaseModel):
    type: Literal["json_match"] = Field(default="json_match")
    expected: Dict[str, Any]

    def eval_action(self, _: str) -> bool:
        # We don't care about action level evaluations
        return True

    def eval_results(self, result: Dict[str, Any]) -> bool:
        return result == self.expected


class ActionEval(BaseModel):
    actions: Dict[str, str]


class Example(BaseModel):
    id: str
    url: str
    source: Literal["mhtml", "hosted"] = Field(description="Source of the website")
    domain: str = Field(description="Domain of the website")
    subdomain: str = Field(description="Subdomain of the website")
    type: GoalType = Field(
        description="The high level type of goal the agent is aiming to do"
    )
    goal: Union[str, Dict[str, Any]] = Field(
        description="The goal of the agent for this specific example"
    )
    fetch_id: Optional[Literal["job_posting", "manufacturing_commerce"]] = Field(
        default=None,
        description="If it is a fetch type, we can infer the goal based on this id to avoid large schemas in json",
    )
    evals: List[Union[JSONEval]] = Field(
        description="Various evaluations to test for within the example"
    )

    def get_static_url(self) -> str:
        from bananalyzer.runner.website_responder import get_website_responder

        return get_website_responder(self).get_url(self)

    @model_validator(mode="before")
    def set_goal_if_fetch_id_provided(cls, values: Dict[str, Any]) -> Dict[str, Any]:
        goal_type = values.get("type")
        if goal_type != "fetch":
            return values

        fetch_id = values.get("fetch_id")
        if fetch_id is None:
            print("test")
            raise ValueError("fetch_id must be provided for fetch goal types")

        goal = values.get("goal")
        if goal is not None:
            raise ValueError("goal must not be provided if fetch_id is provided")

        values["goal"] = fetch_schemas[fetch_id]
        return values
